/*****
* toolkit.h : XmHTML Motif/Gtk function & data type wrappers
*
* This file Version	$Revision: 1.2 $
*
* Creation date:		Thu Jan  8 04:32:19 GMT+0100 1998
* Last modification: 	$Date: 1998/04/27 07:03:52 $
* By:					$Author: newt $
* Current State:		$State: Exp $
*
* Author:				newt
*
* Copyright (C) 1994-1997 by Ripley Software Development 
* All Rights Reserved
*
* This file is part of the XmHTML Widget Library
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
*
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the Free
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* About this file:
*
*	XmHTML is originally an Xt/Motif based Widget. To make porting to
*	other toolkits a bit easier, XmHTML uses a toolkit abstraction and a 
*	set of defines that allow you to replace all X/Xt/Xm functions used by
*	XmHTML.
*
*	All Xt/Xm functions are wrapped together in what I call a
*	ToolkitAbstraction (see XmHTMLP.h for the definition and motif.c for
*	the Motif ToolkitAbstraction).
*
*	There is one assumption though: that you never include a header containing
*	Xt or Xm specifics. If you do this however, you will need to override
*	a whole bunch of routines, typedefs and constants (don't worry, they
*	are all listed).
*
*****/
/*****
* $Source: /usr/local/rcs/Newt/XmHTML/RCS/toolkit.h,v $
*****/
/*****
* ChangeLog 
* $Log: toolkit.h,v $
* Revision 1.2  1998/04/27 07:03:52  newt
* more tka stuff
*
* Revision 1.1  1998/04/04 06:27:29  newt
* Initial Revision
*
*****/ 

#ifndef _toolkit_h_
#define _toolkit_h_

/*****
* default mode = X/Motif
* Add a !defined(WITH_XX) if the toolkit is unlisted and add an appropriate
* set of defines after the last toolkit.
*****/
#if !defined(WITH_GTK)

#define XmHTMLPrivateHeader "XmHTMLP.h"
#define HAVE_XCCP_H

/*****
* X types & constants
* We need defines for these as typedefs would lead to a tremendous amount
* of warnings (with most compilers that is). Lint would go absolutely crazy...
*****/
#define XCOLOR					XColor
#define XCOLOR_WITH_FLAGS		/* XColor has a flags field */

/* TkaColor access macros */
#define GETP(c)	(c).pixel
#define GETR(c)	(c).red
#define GETG(c)	(c).green
#define GETB(c)	(c).blue
#define GETF(c)	(c).flags

#define GETPP(c)	(c)->pixel
#define GETPR(c)	(c)->red
#define GETPG(c)	(c)->green
#define GETPB(c)	(c)->blue
#define GETPF(c)	(c)->flags

/* common types */
#define COLORMAP				Colormap
#define PIXEL					Pixel
#define PIXMAP					Pixmap
#define WINDOW					Window
#define XIMAGE					XImage
#define XFONTSTRUCT				XFontStruct
#define VISUAL					Visual
#define XEVENT					XEvent
#define XBUTTONPRESSEDEVENT		XButtonPressedEvent
#define XBUTTONRELEASEDEVENT	XButtonReleasedEvent

#define XGC							GC

/* byte ordering for this host */
#define LSBFIRST					LSBFirst
#define MSBFIRST					MSBFirst

/*****
* This macro should return a XFontStruct
*****/
#define FontIsXFont(font)			font

#define XTEVENTPROC(f,a,b,c,d) \
	f(a,b,c,d)

/*****
* X Function Wrappers
*****/
#define TkaDefaultRootWindow(dpy) \
	DefaultRootWindow(dpy)

#define TkaWidthOfScreen(widget) \
	WidthOfScreen(XtScreen((Widget)widget))

#define TkaHeightOfScreen(widget) \
	HeightOfScreen(XtScreen((Widget)widget))

#define TkaCurrentTime \
	CurrentTime

/*****
* XFontStruct access macro's
*****/
#define TkaFontLeftBearing(XF)	(XF)->max_bounds.lbearing;
#define TkaFontRightBearing(XF)	(XF)->max_bounds.rbearing;
#define TkaFontWidth(XF)		(XF)->max_bounds.width;
#define TkaFontMaxAscent(XF)	(XF)->max_bounds.ascent;
#define TkaFontMaxDescent(XF)	(XF)->max_bounds.descent;
#define TkaFontLineheight(XF)	((XF)->ascent + (XF)->descent)

/*****
* XImage wrappers & access macros.
*****/

#define TkaImageData(image) \
	(image->data)

#define TkaImageBitsPerPixel(image) \
	(image->bits_per_pixel)

#define TkaImageBytesPerLine(image) \
	(image->bytes_per_line)

#define TkaImageByteOrder(image) \
	(image->byte_order)

#define TkaImageBitmapBitOrder(image) \
	(image->bitmap_bit_order)

/* check support for various combinations of bits per pixel */
#define TkaImageCheck2bpp(image) \
	(image->bits_per_pixel == 2)

#define TkaImageCheck4bpp(image) \
	(image->bits_per_pixel == 4)

#define TkaImageCheck24bpp(image) \
	(image->bits_per_pixel == 24)

#define TkaImageCheck32bpp(image) \
	(image->bits_per_pixel == 32)

/*****
* The next two macros should verify whether or not the *parent* of
* the given widget is of the correct class. If not, they should return
* immediatly.
* The first macro just returns, the second macro returns a value.
*****/
#define CHECK_CLASS(widget,class) do { \
	if(XtClass(XtParent(widget)) != class) \
		return; \
}while(0)

#define CHECK_CLASS_WITH_RETURN(widget,class,retval) do { \
	if(XtClass(XtParent(widget)) != class) \
		return(retval); \
}while(0)

/*****
* This macro evaluates whether or not a callback is installed.
*****/
#define CHECK_CALLBACK(widget,func,signaltype) \
	((widget)->html.func)

/*****
* Widget internal access wrappers
*****/

/* Main access method. *REQUIRES* a variable of html to be known */
#define HTML_ATTR(field)	((XmHTMLWidget)html)->html.field
#define CORE_ATTR(field)	((Widget)html)->core.field
#define MGR_ATTR(field)		((XmHTMLWidget)html)->manager.field

/* subclass access methods */
#define ATTR_CORE(widget,field) \
	(widget)->core.field

#define ATTR_MGR(widget,field) \
	(widget)->manager.field

#define ATTR_HTML(widget,field) \
	((XmHTMLWidget)widget)->html.field

/* widely used subclass properties */

/* get XFont used by the default XmHTMLFont */
#define GetDefaultXFont(widget) \
	ATTR_HTML(widget,default_font->xfont)

#define GetBackground(widget) \
	ATTR_CORE(widget,background_pixel)

#define GetColormap(widget) ATTR_CORE(widget,colormap)

#define VisualGetDepth(widget) \
	ATTR_HTML(widget,xcc->visualInfo->depth)

#define VisualGetMapEntries(visual) \
	visual->map_entries

#else

/* gtk toolkit? */
#if defined(WITH_GTK)

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

/* Private Header to be included for Gtk */
#define XmHTMLPrivateHeader "gtk-xmhtml-p.h"

/* No illegal xt access 'cause we're not using xt at all! */
#ifndef NO_XLIB_ILLEGAL_ACCESS
#define NO_XLIB_ILLEGAL_ACCESS 1
#endif

#define XXEvent					GdkEvent
#define XXButtonPressedEvent	GdkEventButton
#define XXButtonReleasedEvent	GdkEventButton

#include <X11/Xlib.h>

/*
* XPoint and GdkPoint are equivalent, xpoint has short int and gdkpoint
* has gint16
*/

typedef GdkFont     XFONTSTRUCT;
typedef GdkCursor   *XCURSOR;
typedef GdkVisual   VISUAL;
typedef GdkImage    XIMAGE;
typedef GdkColormap *COLORMAP;
typedef GdkPixmap   *PIXMAP;
typedef GdkWindow   *WINDOW;
typedef GdkDrawable *DRAWABLE;
typedef GdkGC       *XGC;
typedef GdkAtom     *XAtom;
typedef GdkColor    XCOLOR;
typedef XVisualInfo XVISUALINFO;

/*****
* Byte ordering used by images
*****/
#define LSBFIRST						GDK_LSB_FIRST
#define MSBFIRST						GDK_MSB_FIRST

/*****
* Provide (or override) a number of Xt typedefs
*****/
#ifdef _XtIntrinsic_h

/* <X11/Intrinsic.h> was included..., define instead of typedef */

#define Widget			GtkWidget*
#define WidgetList		GtkWidget**
#define XtAppContext	gpointer
#define XtCallbackList	GList
#define XtIntervalId	gint
#define XtPointer		gpointer
#define String			char*
#define Boolean			gboolean
#define Cardinal		guint
#define Dimension		gushort
#define Position		gshort

#else

typedef GtkWidget		*Widget;
typedef GtkWidget		**WidgetList;
typedef gpointer		XtAppContext;
typedef GList			XtCallbackList;
typedef gint			XtIntervalId;
typedef gpointer		XtPointer;
typedef char			*String;
typedef gboolean		Boolean
typedef guint			Cardinal;
typedef gushort			Dimension;
typedef gshort			Position;

#endif

/* and for proper definition in gtk-xmhtml-p.h */
typedef gpointer		GdkAppContext;

#define XTEVENTPROC(f,a,b,c,d) \
	f(a, b)

/*****
* This macro should return a XFontStruct
*****/
#define FontIsXFont(font)	((XFontStruct *)(((GdkFontPrivate *)font)->xfont))

/*****
* X Function Wrappers
*****/

#define XXDefaultRootWindow(dpy) \
	GDK_ROOT_PARENT()

#define XXWidthOfScreen(widget) \
	gdk_screen_width ()

#define XXHeightOfScreen(widget) \
	gdk_screen_height ()

#define XXCurrentTime \
	GDK_CURRENT_TIME

/*****
* XImage wrappers & access macros.
*****/

#define XIMAGEData(image) \
	(image->mem)

/* GdkImage has bytes per pixel, not bits per pixel */
#define XIMAGEBitsPerPixel(image) \
	(image->bpp*8)

#define XIMAGEBytesPerLine(image) \
	(image->bpl)

#define XIMAGEByteOrder(image) \
	(image->byte_order)

#define XIMAGEBitmapBitOrder(image) \
	(((GdkImagePrivate*)image)->ximage->bitmap_bit_order)

/*****
* check support for various combinations of bits per pixel
* FIXME:
* Gdk misses support for 2 and 4 bits per pixel.
*****/
#define XIMAGECheck2bpp(image) FALSE

#define XIMAGECheck4bpp(image) FALSE

#define XIMAGECheck24bpp(image) (image->bpp == 3)

#define XIMAGECheck32bpp(image) (image->bpp == 4)

/*****
* Xt Function wrappers
*****/

#define XtName(w)				"(unknown)"
#define ClassName(w)			gtk_widget_get_name(w)

#define XtAppWarning(CTX,MSG)	fputs(MSG, stderr)

#define XtWarning(MSG)			fputs(MSG, stderr)

#define XtAppError(CTX,MSG) do{ fputs(MSG,stderr) ; exit(1); }while(0)

#define XtError(MSG) do{ fputs(MSG,stderr) ; exit(1); }while(0)

/* gdk doesn't have the concept of Application Contexts */
#define XtWidgetToApplicationContext(widget) NULL

/*****
* Motif Wrappers
*****/
/*****
* In Motif, XmUpdateDisplay causes all events that are still left on the
* queue to be flushed *AND* to wait until all requests have been delivered.
* I guess an XFlush followed by an XSync will do the same (sort of).
*****/
#define XmUpdateDisplay(widget) do { \
	gdk_flush(); \
}while(0)

/*****
* Unused by gtk.
*****/
#define CHECK_CLASS(widget,class) /* no-op */

#define CHECK_CLASS_WITH_RETURN(widget,class,retval)	/* no-op */

/*****
* This macro evaluates whether or not a signal is installed.
*****/
#define CHECK_CALLBACK(widget,func,signaltype) \
	(gtk_xmhtml_signal_get_handlers \
		(widget, gtk_xmhtml_signals [GTK_XMHTML_##signaltype]))

/*****
* Widget internal access wrappers
*****/
#define HTML_ATTR(field)	((XmHTMLWidget)html)->html.field
#define CORE_ATTR(field)	(GTK_WIDGET(html)->allocation).field
#define MGR_ATTR(field)		((XmHTMLWidget)html)->manager.field

/* subclass access methods */
#define ATTR_CORE(widget,field) \
	(GTK_WIDGET((widget))->allocation).(field)

#define ATTR_MGR(widget,field) \
	((XmHTMLWidget)widget)->manager.field

#define ATTR_HTML(widget,field) \
	((XmHTMLWidget)widget)->html.field

/* widely used subclass properties */

/* get XFont used by the default XmHTMLFont */
#define GetDefaultXFont(widget) \
	((XFontStruct*)((GdkFontPrivate*)((ATTR_HTML(widget,default_font))->xfont))

#define GetColormap(widget) gtk_widget_get_colormap (GTK_WIDGET (widget))

#define VisualGetDepth(widget) \
	ATTR_HTML(widget,xcc->visual->depth)

#define VisualGetMapEntries(visual) \
	GDK_VISUAL_XVISUAL(visual)->map_entries

/*****
* XColorContext Wrappers
*****/

#define XCCCreate(w,v,c) \
	gdk_color_context_new (v, c)

#define XCCFree(c) \
	gdk_color_context_free (c)

#define XCCGetDepth(c) \
	(c)->visual->depth

#define XCCGetNumColors(c) \
	(c)->num_colors

#define XCCGetParentVisual(w) \
	gtk_widget_get_visual(w)

#define XCCGetPixels(cc,r,g,b,n,co,a) \
	gdk_color_context_get_pixels (cc,r,g,b,n,co,a)

#define XCCGetPixelsIncremental(cc,r,g,b,n,u,co,na) do{ \
	gdk_color_context_get_pixels_incremental (cc,r,g,b,n,u,co,na); \
}while (0)

#define XCCAddPalette(c,p,n) \
	gdk_color_context_add_palette (c,p,n)

#define XCCInitDither(cc) \
	gdk_color_context_init_dither (cc)

#define XCCGetIndexFromPalette(cc,r,g,b,f) \
	gdk_color_context_get_index_from_palette(cc,r,g,b,f)

#define XCCFreeDither(cc) \
	gdk_color_context_free_dither (cc)

typedef GdkColorContextDither XCCDither;

#endif /* WITH_GTK */

/*****
* Add a set of defines for your toolkit here
*****/

#endif /* outer wrapper */

/* Don't add anything after this endif! */
#endif /* _toolkit_h_ */
